local scroll_speed = 1.5  -- Speed of scrolling
local scroll_speed_multiplier = 3  -- Multiplier for the scrolling speed
local scroll_duration = 5  -- Duration in seconds for how long the scrolling stays active

local scroll_state = {}  -- Stores the current state of the player

-- Initialize the player's state if it doesn't exist
local function ensure_player_state(player_index)
  if not scroll_state[player_index] then
    scroll_state[player_index] = {up=false, down=false, left=false, right=false, timer=0}
  end
end

-- Event handler for key presses
local function handle_scroll_event(event, pressed)
  local name = event.input_name
  local player_index = event.player_index
  ensure_player_state(player_index)

  -- Check if the input is one of the scroll keys
  if name == "fast-scroll-up" then scroll_state[player_index].up = pressed end
  if name == "fast-scroll-down" then scroll_state[player_index].down = pressed end
  if name == "fast-scroll-left" then scroll_state[player_index].left = pressed end
  if name == "fast-scroll-right" then scroll_state[player_index].right = pressed end
end

-- Event for handling input events
script.on_event(defines.events.on_input_event, function(event)
  if event.input_name then
    -- Only respond to custom input keys
    if event.input_name == "fast-scroll-up" or event.input_name == "fast-scroll-down" or
       event.input_name == "fast-scroll-left" or event.input_name == "fast-scroll-right" then
      -- Handle the input
      handle_scroll_event(event, event.state == defines.input_state.pressed)
    end
  end
end)

-- Tick-based movement (called every tick)
script.on_event(defines.events.on_tick, function()
  for player_index, state in pairs(scroll_state) do
    local player = game.get_player(player_index)
    if player and player.valid and player.controller_type == defines.controllers.god then
      local dx, dy = 0, 0

      -- Movement based on pressed keys
      if state.up then dy = dy - (scroll_speed * scroll_speed_multiplier) end
      if state.down then dy = dy + (scroll_speed * scroll_speed_multiplier) end
      if state.left then dx = dx - (scroll_speed * scroll_speed_multiplier) end
      if state.right then dx = dx + (scroll_speed * scroll_speed_multiplier) end

      -- Apply the movement
      if dx ~= 0 or dy ~= 0 then
        local pos = player.position
        player.teleport({pos.x + dx, pos.y + dy})

        -- Increment the timer to stop scrolling after a set duration
        scroll_state[player_index].timer = scroll_state[player_index].timer + 1
      end

      -- Stop the movement after a certain time
      if scroll_state[player_index].timer >= scroll_duration then
        scroll_state[player_index].up = false
        scroll_state[player_index].down = false
        scroll_state[player_index].left = false
        scroll_state[player_index].right = false
        scroll_state[player_index].timer = 0  -- Reset the timer
      end
    end
  end
end)
